/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2021 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/bits/bits.h"
#include "libc/bits/safemacros.internal.h"
#include "libc/calls/calls.h"
#include "libc/calls/syscall-sysv.internal.h"
#include "libc/dce.h"
#include "libc/errno.h"
#include "libc/intrin/kprintf.h"
#include "libc/mem/alloca.h"
#include "libc/paths.h"
#include "libc/runtime/runtime.h"
#include "libc/str/str.h"
#include "libc/sysv/consts/at.h"
#include "libc/sysv/consts/o.h"
#include "libc/sysv/consts/ok.h"
#include "libc/sysv/errfuns.h"

static bool CanExecute(const char *path) {
  return !sys_faccessat(AT_FDCWD, path, X_OK, 0);
}

static bool IsApeBinary(const char *path) {
  int fd;
  char buf[8];
  bool res = false;
  if ((fd = sys_open(path, O_RDONLY, 0)) != -1) {
    if (sys_read(fd, buf, 8) == 8 && READ64LE(buf) == READ64LE("MZqFpD='")) {
      res = true;
    }
    sys_close(fd);
  }
  return res;
}

static const char *Join(const char *a, const char *b, char buf[PATH_MAX]) {
  size_t n, m;
  n = strlen(a);
  m = strlen(b);
  if (n + 1 + m + 1 < PATH_MAX) {
    stpcpy(stpcpy(stpcpy(buf, a), "/"), b);
    return buf;
  } else {
    return "";
  }
}

int sys_execve(const char *prog, char *const argv[], char *const envp[]) {
  int e;
  size_t i;
  char *buf;
  char **shargs;
  const char *ape;
  e = errno;
  __sys_execve(prog, argv, envp);
  if (errno != ENOEXEC) return -1;
  for (i = 0; argv[i];) ++i;
  buf = alloca(PATH_MAX);
  shargs = alloca((i + 4) * sizeof(char *));
  if (IsApeBinary(prog) &&
      (CanExecute((ape = "/usr/bin/ape")) ||
       CanExecute((ape = Join(firstnonnull(getenv("TMPDIR"),
                                           firstnonnull(getenv("HOME"), ".")),
                              ".ape", buf))))) {
    shargs[0] = ape;
    shargs[1] = "-";
    shargs[2] = prog;
    memcpy(shargs + 3, argv, (i + 1) * sizeof(char *));
  } else if (CanExecute(prog)) {
    shargs[0] = _PATH_BSHELL;
    shargs[1] = prog;
    memcpy(shargs + 2, argv + 1, i * sizeof(char *));
  } else {
    return enoexec();
  }
  errno = e;
  return __sys_execve(shargs[0], shargs, envp);
}
